<?php

namespace Drupal\commerce_recurly;

use Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher;
use Drupal\recurly\Event\notifications\account\RecurlyBillingInfoUpdatedEvent;
use Symfony\Component\EventDispatcher\Event;

/**
 * Helps to determine which events should be dispatched for notifications.
 *
 * @package Drupal\recurly
 */
class RecurlyWebhookEventDispatcher implements RecurlyWebhookEventDispatcherInterface {

  /**
   * The event dispatcher.
   *
   * @var \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher
   */
  protected $eventDispatcher;

  /**
   * RecurlyWebhookEventDispatcher constructor.
   *
   * @param \Drupal\Component\EventDispatcher\ContainerAwareEventDispatcher $event_dispatcher
   *   The event dispatcher.
   */
  public function __construct(ContainerAwareEventDispatcher $event_dispatcher) {
    $this->eventDispatcher = $event_dispatcher;
  }

  /**
   * {@inheritDoc}
   */
  public function dispatchEvent(\Recurly_PushNotification $notification, string $subdomain): Event {
    $event_name_const_lower = str_replace('_notification', '', $notification->type);
    $event_const_name = strtoupper($event_name_const_lower);

    return $this->eventDispatcher->dispatch(
      constant("RecurlyWebhookEvents::$event_const_name"),
      new RecurlyBillingInfoUpdatedEvent($notification, $subdomain)
    );
  }

}
